=== modified file 'NEWS'
--- NEWS	2007-06-01 23:10:18 +0000
+++ NEWS	2007-06-04 18:59:21 +0000
@@ -47,6 +47,9 @@
     * WorkingTree4.get_file_sha1 no longer raises an exception when invoked
       on a missing file.  (Aaron Bentley, #118186)
 
+    * WorkingTree.remove works correctly with tree references, and when pwd is
+      not the tree root. (Aaron Bentley)
+
 bzr 0.16  2007-05-07
   
   BUGFIXES:

=== modified file 'bzrlib/tests/workingtree_implementations/test_remove.py'
--- bzrlib/tests/workingtree_implementations/test_remove.py	2007-04-18 20:12:15 +0000
+++ bzrlib/tests/workingtree_implementations/test_remove.py	2007-05-24 12:19:41 +0000
@@ -203,3 +203,18 @@
         tree.remove(TestRemove.b, keep_files=False, force=True)
         self.assertNotInWorkingTree(TestRemove.b_c)
         self.failIfExists(TestRemove.b_c)
+
+    def test_remove_subtree(self):
+        tree = self.make_branch_and_tree('.')
+        subtree = self.make_branch_and_tree('subtree')
+        tree.add('subtree', 'subtree-id')
+        tree.remove('subtree')
+        self.assertIs(None, tree.path2id('subtree'))
+
+    def test_non_cwd(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/dir/', 'tree/dir/file'])
+        tree.add(['dir', 'dir/file'])
+        tree.commit('add file')
+        tree.remove('dir/', keep_files=False)
+        self.failIfExists('tree/dir/file')

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-05-02 14:54:20 +0000
+++ bzrlib/workingtree.py	2007-05-24 15:36:19 +0000
@@ -1775,7 +1775,7 @@
         if isinstance(files, basestring):
             files = [files]
 
-        inv = self.inventory
+        inv_delta = []
 
         new_files=set()
         unknown_files_in_directory=set()
@@ -1783,22 +1783,24 @@
         def recurse_directory_to_add_files(directory):
             # recurse directory and add all files
             # so we can check if they have changed.
-            for contained_dir_info in self.walkdirs(directory):
-                for file_info in contained_dir_info[1]:
-                    if file_info[2] == 'file':
-                        relpath = self.relpath(file_info[0])
-                        if file_info[4]: #is it versioned?
+            for parent_info, file_infos in\
+                osutils.walkdirs(self.abspath(directory),
+                    directory):
+                for relpath, basename, kind, lstat, abspath in file_infos:
+                    if kind == 'file':
+                        if self.path2id(relpath): #is it versioned?
                             new_files.add(relpath)
                         else:
                             unknown_files_in_directory.add(
-                                (relpath, None, file_info[2]))
+                                (relpath, None, kind))
 
         for filename in files:
             # Get file name into canonical form.
-            filename = self.relpath(self.abspath(filename))
+            abspath = self.abspath(filename)
+            filename = self.relpath(abspath)
             if len(filename) > 0:
                 new_files.add(filename)
-                if osutils.isdir(filename) and len(os.listdir(filename)) > 0:
+                if osutils.isdir(abspath):
                     recurse_directory_to_add_files(filename)
         files = [f for f in new_files]
 
@@ -1818,7 +1820,7 @@
 
         # do this before any modifications
         for f in files:
-            fid = inv.path2id(f)
+            fid = self.path2id(f)
             message=None
             if not fid:
                 message="%s is not versioned." % (f,)
@@ -1829,10 +1831,10 @@
                         new_status = 'I'
                     else:
                         new_status = '?'
-                    textui.show_status(new_status, inv[fid].kind, f,
+                    textui.show_status(new_status, self.kind(fid), f,
                                        to_file=to_file)
                 # unversion file
-                del inv[fid]
+                inv_delta.append((f, None, fid, None))
                 message="removed %s" % (f,)
 
             if not keep_files:
@@ -1852,7 +1854,7 @@
             # print only one message (if any) per file.
             if message is not None:
                 note(message)
-        self._write_inventory(inv)
+        self.apply_inventory_delta(inv_delta)
 
     @needs_tree_write_lock
     def revert(self, filenames, old_tree=None, backups=True, 

